@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 
 import numpy as np
-from roboticstoolbox.robot.ERobot import ERobot
+from roboticstoolbox.robot.Robot import Robot
 
 
-class rx150(ERobot):
+class rx150(Robot):
     """
     Class that imports a RX150 URDF model
 
@@ -24,29 +24,34 @@ class rx150(ERobot):
     - qr, vertical 'READY' configuration
 
     :reference:
-        - http://www.support.interbotix.com/html/specifications/rx150.html
+        - https://docs.trossenrobotics.com/interbotix_xsarms_docs/specifications/rx150.html
 
     .. codeauthor:: Jesse Haviland
     .. sectionauthor:: Peter Corke
     """
+
     def __init__(self):
 
-        links, name = self.URDF_read(
-            "interbotix_descriptions/urdf/rx150.urdf.xacro")
+        links, name, urdf_string, urdf_filepath = self.URDF_read(
+            "interbotix_descriptions/urdf/rx150.urdf.xacro"
+        )
 
         super().__init__(
-                links,
-                name=name,
-                manufacturer='Interbotix'
-            )
+            links,
+            name=name,
+            manufacturer="Interbotix",
+            urdf_string=urdf_string,
+            urdf_filepath=urdf_filepath,
+        )
 
-        self.addconfiguration(
-            "qz", np.array([0, 0, 0, 0, 0, 0, 0, 0]))
-        self.addconfiguration(
-            "qr", np.array([0, -0.3, 0, -2.2, 0, 2.0, np.pi/4, 0]))
+        self.qr = np.array([0, -0.3, 0, -2.2, 0, 2.0, np.pi / 4, 0])
+        self.qz = np.zeros(7)
+
+        self.addconfiguration("qr", self.qr)
+        self.addconfiguration("qz", self.qz)
 
 
-if __name__ == '__main__':   # pragma nocover
+if __name__ == "__main__":  # pragma nocover
 
     robot = rx150()
     print(robot)
