@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 
 import numpy as np
-from roboticstoolbox.robot.ERobot import ERobot
+from roboticstoolbox.robot.Robot import Robot
 
 
-class wx250s(ERobot):
+class wx250s(Robot):
     """
     Class that imports a wx250s URDF model
 
@@ -24,7 +24,7 @@ class wx250s(ERobot):
     - qr, vertical 'READY' configuration
 
     :reference:
-        - http://www.support.interbotix.com/html/specifications/wx2506dof.html
+        - https://docs.trossenrobotics.com/interbotix_xsarms_docs/specifications/wx250s.html
 
     .. codeauthor:: Jesse Haviland
     .. sectionauthor:: Peter Corke
